unit avisynth_c;

{$ALIGN 8}

interface

type
  PAVS_VideoFrameBuffer = ^AVS_VideoFrameBuffer;
  PAVS_VideoFrame = ^AVS_VideoFrame;
  PAVS_ScriptEnvironment = type Pointer;
  PAVS_Clip = type Pointer;
  PAVS_Value = ^AVS_Value;
  PAVS_FilterInfo = ^AVS_FilterInfo;
  PAVS_VideoInfo = ^AVS_VideoInfo;
  PAVS_Value_array = ^AVS_Value_array;
  MSVC_AVS_Value = type Int64;

  Planes = (
    AVS_NOT_PLANAR,
    AVS_PLANAR_Y,
    AVS_PLANAR_U,
    AVS_PLANAR_V,
    AVS_PLANAR_Y_ALIGNED,
    AVS_PLANAR_U_ALIGNED,
    AVS_PLANAR_V_ALIGNED);

  AVS_VideoInfo = record
    width, height: Integer;
    fps_numerator, fps_denominator: Cardinal;
    num_frames: Integer;
    pixel_type: Integer;
    audio_samples_per_second: Integer;
    sample_type: Integer;
    num_audio_samples: Int64;
    nchannels: Integer;
    image_type: Integer;
  end;

  AVS_VideoFrameBuffer = record
    data: PByte;
    data_size: Integer;
    sequence_number: Cardinal;
    refcount: Cardinal;
  end;

  AVS_VideoFrame = record
    refcount: Integer;
    vfb: PAVS_VideoFrameBuffer;
    offset, pitch, row_size, height, offsetU, offsetV, pitchUV: Integer;
  end;

  AVS_Value = record
    vtype: WideChar;
    array_size: Smallint;
    case char of
      'c': (vclip: Pointer);
      'b': (vboolean: LongBool);
      'i': (vinteger: Integer);
      'f': (vfloating_pt: Single);
      's': (vstring: PChar);
      'a': (varray: PAVS_Value_array);
  end;

  AVS_Value_array = array[0..50] of AVS_Value;

  get_frame_func = function(fi: PAVS_FilterInfo; n: Integer): PAVS_VideoFrame; stdcall;
  get_parity_func = function(fi: PAVS_FilterInfo; n: Integer): Integer; stdcall;
  get_audio_func = function(fi: PAVS_FilterInfo; buf: Pointer; start: Int64; count: Int64): Integer; stdcall;
  set_cache_hints_func = function(fi: PAVS_FilterInfo; cachehints: Integer; frame_range: Integer): Integer; stdcall;
  free_filter_func = procedure(fi: PAVS_FilterInfo); stdcall;

  AVS_ApplyFunc = function(env: PAVS_ScriptEnvironment; args: MSVC_AVS_Value; user_data: Pointer): MSVC_AVS_Value; stdcall;
  AVS_ShutdownFunc = procedure(user_data: Pointer; env: PAVS_ScriptEnvironment); stdcall;

  AVS_FilterInfo = record
    child: PAVS_Clip;
    vi: AVS_VideoInfo;
    env: PAVS_ScriptEnvironment;

    get_frame: get_frame_func;
    get_parity: get_parity_func;
    get_audio_func: get_audio_func;
    set_cache_hints_func: set_cache_hints_func;
    free_filter: free_filter_func;

    error: PChar;
    user_data: Pointer;
  end;

const
  AVS_CPU_FORCE = $01;
  AVS_CPU_FPU = $02;
  AVS_CPU_MMX = $04;
  AVS_CPU_INTEGER_SSE = $08;
  AVS_CPU_SSE = $10;
  AVS_CPU_SSE2 = $20;
  AVS_CPU_3DNOW = $40;
  AVS_CPU_3DNOW_EXT = $80;
  AVS_CPU_X86_64 = $A0;

  AVS_SAMPLE_INT8 = 1 shl 0;
  AVS_SAMPLE_INT16 = 1 shl 1;
  AVS_SAMPLE_INT24 = 1 shl 2;
  AVS_SAMPLE_INT32 = 1 shl 3;
  AVS_SAMPLE_FLOAT = 1 shl 4;

  // Colorspace properties.
  AVS_CS_BGR = 1 shl 28;
  AVS_CS_YUV = 1 shl 29;
  AVS_CS_INTERLEAVED = 1 shl 30;
  AVS_CS_PLANAR = 1 shl 31;
  AVS_CS_UNKNOWN = 0;

  // Specific colorformats
  AVS_CS_BGR24 = 1 shl 0 or AVS_CS_BGR or AVS_CS_INTERLEAVED;
  AVS_CS_BGR32 = 1 shl 1 or AVS_CS_BGR or AVS_CS_INTERLEAVED;
  AVS_CS_YUY2 = 1 shl 2 or AVS_CS_YUV or AVS_CS_INTERLEAVED;
  AVS_CS_YV12 = 1 shl 3 or AVS_CS_YUV or AVS_CS_PLANAR;
  AVS_CS_I420 = 1 shl 4 or AVS_CS_YUV or AVS_CS_PLANAR;
  AVS_CS_IYUV = 1 shl 4 or AVS_CS_YUV or AVS_CS_PLANAR;

  AVS_IT_BFF = 1 shl 0;
  AVS_IT_TFF = 1 shl 1;
  AVS_IT_FIELDBASED = 1 shl 2;

  AVS_FILTER_TYPE = 1;
  AVS_FILTER_INPUT_COLORSPACE = 2;
  AVS_FILTER_OUTPUT_TYPE = 9;
  AVS_FILTER_NAME = 4;
  AVS_FILTER_AUTHOR = 5;
  AVS_FILTER_VERSION = 6;
  AVS_FILTER_ARGS = 7;
  AVS_FILTER_ARGS_INFO = 8;
  AVS_FILTER_ARGS_DESCRIPTION = 10;
  AVS_FILTER_DESCRIPTION = 11;
  AVS_FILTER_TYPE_AUDIO = 1;
  AVS_FILTER_TYPE_VIDEO = 2;
  AVS_FILTER_OUTPUT_TYPE_SAME = 3;
  AVS_FILTER_OUTPUT_TYPE_DIFFERENT = 4;

  AVS_CACHE_NOTHING = 0;
  AVS_CACHE_RANGE = 1;
  AVS_CACHE_ALL = 2;
  AVS_CACHE_AUDIO = 3;
  AVS_CACHE_AUDIO_NONE = 4;

  AVISYNTH_INTERFACE_VERSION = 2;
  AVS_FRAME_ALIGN = 16;

  avs_no_arguments: AVS_Value = (vtype: 'a'; array_size: 0; vinteger: 0);
  avs_void: AVS_Value = (vtype: 'v'; array_size: 0; vinteger: 0);

// Imported functions
function avs_create_script_environment(version: Integer = AVISYNTH_INTERFACE_VERSION): PAVS_ScriptEnvironment; stdcall; external 'avisynth.dll';
procedure avs_set_to_clip(var v: AVS_Value; clip: PAVS_Clip); stdcall; external 'avisynth.dll';
function avs_new_video_frame(env: PAVS_ScriptEnvironment; vi: PAVS_VideoInfo; align: Integer = AVS_FRAME_ALIGN): PAVS_VideoFrame; stdcall; external 'avisynth.dll' name 'avs_new_video_frame_a';
procedure avs_release_video_frame(pvf: PAVS_VideoFrame); stdcall; external 'avisynth.dll';
function avs_copy_video_frame(pvf: PAVS_VideoFrame): PAVS_VideoFrame; stdcall; external 'avisynth.dll';
procedure avs_release_clip(pac: PAVS_Clip); stdcall; external 'avisynth.dll';
function avs_copy_clip(pac: PAVS_Clip): PAVS_Clip; stdcall; external 'avisynth.dll';
function avs_clip_get_error(pac: PAVS_Clip): PChar; stdcall; external 'avisynth.dll';
function avs_get_video_info(pac: PAVS_Clip): PAVS_VideoInfo; stdcall; external 'avisynth.dll';
function avs_get_version(pac: PAVS_Clip): Integer; stdcall; external 'avisynth.dll';
function avs_get_frame(pac: PAVS_Clip; n: Integer): PAVS_VideoFrame; stdcall; external 'avisynth.dll';
function avs_get_parity(pac: PAVS_Clip; n: Integer): Integer; stdcall; external 'avisynth.dll';
function avs_get_audio(pac: PAVS_Clip; buf: Pointer; start: Int64; count: Int64): PAVS_Clip; stdcall; external 'avisynth.dll';
function avs_set_cache_hints(pac: PAVS_Clip; cachehints: Integer; frame_range: Integer): Integer; stdcall; external 'avisynth.dll';
function avs_get_cpu_flags(env: PAVS_ScriptEnvironment): Cardinal; stdcall; external 'avisynth.dll';
function avs_check_version(env: PAVS_ScriptEnvironment; version: Integer): LongBool; stdcall; external 'avisynth.dll';
function avs_save_string(env: PAVS_ScriptEnvironment; s: PChar; length: Integer): PChar; stdcall; external 'avisynth.dll';
function avs_add_function(env: PAVS_ScriptEnvironment; name: PChar; params: PChar; apply: AVS_ApplyFunc; user_data: Pointer): Integer; stdcall; external 'avisynth.dll';
function avs_function_exists(env: PAVS_ScriptEnvironment; name: PChar): LongBool; stdcall; external 'avisynth.dll';
function avs_make_writable(env: PAVS_ScriptEnvironment; var pvf: PAVS_VideoFrame): Integer; stdcall; external 'avisynth.dll';
procedure avs_bit_blt(env: PAVS_ScriptEnvironment; dstp: PByte; dst_pitch: Integer; srcp: PByte; src_pitch: Integer; row_size: Integer; height: Integer); stdcall; external 'avisynth.dll';
procedure avs_at_exit(env: PAVS_ScriptEnvironment; sfunction: AVS_ShutdownFunc; user_data: Pointer); stdcall; external 'avisynth.dll';
function avs_subframe(env: PAVS_ScriptEnvironment; src: PAVS_VideoFrame; rel_offset: Integer; new_pitch: Integer; new_row_size: Integer; new_height: Integer): PAVS_VideoFrame; stdcall; external 'avisynth.dll';
function avs_set_memory_max(env: PAVS_ScriptEnvironment; mem: Integer): Integer; stdcall; external 'avisynth.dll';
function avs_set_working_dir(env: PAVS_ScriptEnvironment; newdir: PChar): LongBool; stdcall; external 'avisynth.dll';

function avs_new_c_filter(e: PAVS_ScriptEnvironment; var fi: PAVS_FilterInfo; child: MSVC_AVS_Value; store_child: Integer): PAVS_Clip; stdcall; external 'avisynth.dll';

// All local funtions
procedure avs_copy_value(var dest: AVS_Value; src: AVS_Value);
procedure avs_release_value(v: AVS_Value);
function avs_take_clip(v: AVS_Value; env: PAVS_ScriptEnvironment): PAVS_Clip;
function avs_invoke(env: PAVS_ScriptEnvironment; name: PChar; args: AVS_Value; arg_names: PPChar = nil): AVS_Value;
function avs_get_var(env: PAVS_ScriptEnvironment; name: PChar): AVS_Value;
function avs_set_var(env: PAVS_ScriptEnvironment; name: PChar; val: AVS_Value): Boolean;
function avs_set_global_var(env: PAVS_ScriptEnvironment; name: PChar; val: AVS_Value): Boolean;
function avs_has_video(p: PAVS_VideoInfo): Boolean;
function avs_has_audio(p: PAVS_VideoInfo): Boolean;
function avs_is_rgb(p: PAVS_VideoInfo): Boolean;
function avs_is_rgb24(p: PAVS_VideoInfo): Boolean;
function avs_is_rgb32(p: PAVS_VideoInfo): Boolean;
function avs_is_yuy(p: PAVS_VideoInfo): Boolean;
function avs_is_yuy2(p: PAVS_VideoInfo): Boolean;
function avs_is_yv12(p: PAVS_VideoInfo): Boolean;
function avs_is_color_space(p: PAVS_VideoInfo; c_space: Integer): Boolean;
function avs_is_property(p: PAVS_VideoInfo; vproperty: Integer): Boolean;
function avs_is_planar(p: PAVS_VideoInfo): Boolean;
function avs_is_field_based(p: PAVS_VideoInfo): Boolean;
function avs_is_parity_known(p: PAVS_VideoInfo): Boolean;
function avs_is_bff(p: PAVS_VideoInfo): Boolean;
function avs_is_tff(p: PAVS_VideoInfo): Boolean;
function avs_bits_per_pixel(p: PAVS_VideoInfo): Integer;
function avs_bytes_from_pixels(p: PAVS_VideoInfo; pixels: Integer): Integer;
function avs_row_size(p: PAVS_VideoInfo): Integer;
function avs_bmp_size(vi: PAVS_VideoInfo): Integer;
function avs_bytes_per_channel_sample(p: PAVS_VideoInfo): Integer;
function avs_bytes_per_audio_sample(p: PAVS_VideoInfo): Integer;
function avs_audio_samples_from_frames(p: PAVS_VideoInfo; frames: Int64): Int64;
function avs_frames_from_audio_samples(p: PAVS_VideoInfo; samples: Int64): Integer;
function avs_audio_samples_from_bytes(p: PAVS_VideoInfo; bytes: Int64): Int64;
function avs_bytes_from_audio_samples(p: PAVS_VideoInfo; samples: Int64): Int64;
procedure avs_set_property(p: PAVS_VideoInfo; vproperty: Integer);
procedure avs_clear_property(p: PAVS_VideoInfo; vproperty: Integer);
procedure avs_set_fps(p: PAVS_VideoInfo; numerator: Cardinal; denominator: Cardinal);
function avs_is_same_colorspace(x: PAVS_VideoInfo; y: PAVS_VideoInfo): Boolean;
function avs_get_pitch(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): Integer;
function avs_get_row_size(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): Integer;
function avs_get_height(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): Integer;
function avs_get_read_ptr(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): PByte;
function avs_is_writable(p: PAVS_VideoFrame): Boolean;
function avs_get_write_ptr(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): PByte;
function avs_defined(v: AVS_Value): Boolean;
function avs_is_clip(v: AVS_Value): Boolean;
function avs_is_bool(v: AVS_Value): Boolean;
function avs_is_int(v: AVS_Value): Boolean;
function avs_is_float(v: AVS_Value): Boolean;
function avs_is_string(v: AVS_Value): Boolean;
function avs_is_array(v: AVS_Value): Boolean;
function avs_is_error(v: AVS_Value): Boolean;
function avs_as_bool(v: AVS_Value): LongBool;
function avs_as_int(v: AVS_Value): Integer;
function avs_as_string(v: AVS_Value): PChar;
function avs_as_float(v: AVS_Value): Single;
function avs_as_error(v: AVS_Value): PChar;
function avs_as_array(v: AVS_Value): PAVS_Value_array;
function avs_array_size(v: AVS_Value): Integer;
function avs_array_elt(v: AVS_Value; index: Integer): AVS_Value;
function avs_new_value_bool(v0: LongBool): AVS_Value;
function avs_new_value_int(v0: Integer): AVS_Value;
function avs_new_value_string(v0: PChar): AVS_Value;
function avs_new_value_float(v0: Single): AVS_Value;
function avs_new_value_error(v0: PChar): AVS_Value;
function avs_new_value_clip(v0: PAVS_Clip): AVS_Value;
function avs_new_value_array(v0: PAVS_Value_array; size: Integer): AVS_Value;

implementation

// Imported functions using AVS_Value which need special typecasting to work properly
procedure ext_avs_copy_value(var dest: AVS_Value; src: MSVC_AVS_Value); stdcall; external 'avisynth.dll' name 'avs_copy_value';
procedure ext_avs_release_value(v: MSVC_AVS_Value); stdcall; external 'avisynth.dll' name 'avs_release_value';
function ext_avs_take_clip(v: MSVC_AVS_Value; env: PAVS_ScriptEnvironment): PAVS_Clip; stdcall; external 'avisynth.dll' name 'avs_take_clip';
function ext_avs_invoke(env: PAVS_ScriptEnvironment; name: PChar; args: MSVC_AVS_Value; arg_names: PPChar): MSVC_AVS_Value; stdcall; external 'avisynth.dll' name 'avs_invoke';
function ext_avs_get_var(env: PAVS_ScriptEnvironment; name: PChar): MSVC_AVS_Value; stdcall; external 'avisynth.dll' name 'avs_get_var';
function ext_avs_set_var(env: PAVS_ScriptEnvironment; name: PChar; val: MSVC_AVS_Value): LongBool; stdcall; external 'avisynth.dll' name 'avs_set_var';
function ext_avs_set_global_var(env: PAVS_ScriptEnvironment; name: PChar; val: MSVC_AVS_Value): LongBool; stdcall; external 'avisynth.dll' name 'avs_set_global_var';

// Typecasting for visual c++ records
procedure avs_copy_value(var dest: AVS_Value; src: AVS_Value);
begin
  ext_avs_copy_value(dest, MSVC_AVS_Value(src));
end;

procedure avs_release_value(v: AVS_Value);
begin
  ext_avs_release_value(MSVC_AVS_Value(v));
end;

function avs_take_clip(v: AVS_Value; env: PAVS_ScriptEnvironment): PAVS_Clip;
begin
  Result := ext_avs_take_clip(MSVC_AVS_Value(v), env);
end;

function avs_invoke(env: PAVS_ScriptEnvironment; name: PChar; args: AVS_Value; arg_names: PPChar = nil): AVS_Value;
begin
  Result := AVS_Value(ext_avs_invoke(env, name, MSVC_AVS_Value(args), arg_names));
end;

function avs_get_var(env: PAVS_ScriptEnvironment; name: PChar): AVS_Value;
begin
  Result := AVS_Value(ext_avs_get_var(env, name));
end;

function avs_set_var(env: PAVS_ScriptEnvironment; name: PChar; val: AVS_Value): Boolean;
begin
  Result := ext_avs_set_var(env, name, MSVC_AVS_Value(val));
end;

function avs_set_global_var(env: PAVS_ScriptEnvironment; name: PChar; val: AVS_Value): Boolean;
begin
  Result := ext_avs_set_global_var(env, name, MSVC_AVS_Value(val));
end;

///////////////////////////////////////////////////////////

function avs_has_video(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.width <> 0;
end;

function avs_has_audio(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.audio_samples_per_second <> 0;
end;

function avs_is_rgb(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.pixel_type and AVS_CS_BGR <> 0;
end;

function avs_is_rgb24(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.pixel_type and AVS_CS_BGR24 = AVS_CS_BGR24;
end;

function avs_is_rgb32(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.pixel_type and AVS_CS_BGR32 = AVS_CS_BGR32;
end;

function avs_is_yuy(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.pixel_type and AVS_CS_YUV <> 0;
end;

function avs_is_yuy2(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.pixel_type and AVS_CS_YUY2 = AVS_CS_YUY2;
end;

function avs_is_yv12(p: PAVS_VideoInfo): Boolean;
begin
  Result := (p^.pixel_type and AVS_CS_YV12 = AVS_CS_YV12) or (p^.pixel_type and AVS_CS_I420 = AVS_CS_I420);
end;

function avs_is_color_space(p: PAVS_VideoInfo; c_space: Integer): Boolean;
begin
  Result := p^.pixel_type and c_space = c_space;
end;

function avs_is_property(p: PAVS_VideoInfo; vproperty: Integer): Boolean;
begin
  Result := p^.pixel_type and vproperty = vproperty;
end;

function avs_is_planar(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.pixel_type and AVS_CS_PLANAR = AVS_CS_PLANAR;
end;

function avs_is_field_based(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.image_type and AVS_IT_FIELDBASED = AVS_IT_FIELDBASED;
end;

function avs_is_parity_known(p: PAVS_VideoInfo): Boolean;
begin
  Result := (p^.image_type and AVS_IT_FIELDBASED = AVS_IT_FIELDBASED) and (p^.image_type and (AVS_IT_BFF or AVS_IT_TFF) <> 0);
end;

function avs_is_bff(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.image_type and AVS_IT_BFF = AVS_IT_BFF;
end;

function avs_is_tff(p: PAVS_VideoInfo): Boolean;
begin
  Result := p^.image_type and AVS_IT_TFF = AVS_IT_TFF;
end;

function avs_bits_per_pixel(p: PAVS_VideoInfo): Integer;
begin
  case p^.image_type of
    AVS_CS_BGR24: Result := 24;
    AVS_CS_BGR32: Result := 32;
    AVS_CS_YUY2: Result := 16;
    AVS_CS_YV12, AVS_CS_I420: Result := 12;
  else
    Result := 0;
  end;
end;

function avs_bytes_from_pixels(p: PAVS_VideoInfo; pixels: Integer): Integer;
begin
  Result := pixels * (avs_bits_per_pixel(p) shr 3);
end;

function avs_row_size(p: PAVS_VideoInfo): Integer;
begin
  Result := avs_bytes_from_pixels(p, p^.width);
end;

function avs_bmp_size(vi: PAVS_VideoInfo): Integer;
begin
  if avs_is_planar(vi) then
  begin
    Result := vi^.height * ((avs_row_size(vi) + 3) and not 3);
    Inc(Result, Result shr 1);
  end
  else
    Result := vi^.height * ((avs_row_size(vi) + 3) and not 3);
end;

function avs_bytes_per_channel_sample(p: PAVS_VideoInfo): Integer;
begin
  case p^.sample_type of
    AVS_SAMPLE_INT8: Result := 1;
    AVS_SAMPLE_INT16: Result := 2;
    AVS_SAMPLE_INT24: Result := 3;
    AVS_SAMPLE_INT32: Result := 4;
    AVS_SAMPLE_FLOAT: Result := 4;
  else
    Result := 0;
  end;
end;

function avs_bytes_per_audio_sample(p: PAVS_VideoInfo): Integer;
begin
  Result := p^.nchannels * avs_bytes_per_channel_sample(p);
end;

function avs_audio_samples_from_frames(p: PAVS_VideoInfo; frames: Int64): Int64;
begin
  Result := (frames * p^.audio_samples_per_second * p^.fps_denominator) div p^.fps_numerator;
end;

function avs_frames_from_audio_samples(p: PAVS_VideoInfo; samples: Int64): Integer;
begin
  Result := samples * p^.fps_numerator div p^.fps_denominator div p^.audio_samples_per_second;
end;

function avs_audio_samples_from_bytes(p: PAVS_VideoInfo; bytes: Int64): Int64;
begin
  Result := bytes div avs_bytes_per_audio_sample(p);
end;

function avs_bytes_from_audio_samples(p: PAVS_VideoInfo; samples: Int64): Int64;
begin
  Result := samples * avs_bytes_per_audio_sample(p);
end;

procedure avs_set_property(p: PAVS_VideoInfo; vproperty: Integer);
begin
  p^.image_type := p^.image_type or vproperty;
end;

procedure avs_clear_property(p: PAVS_VideoInfo; vproperty: Integer);
begin
  p^.image_type := p^.image_type and not vproperty;
end;

procedure avs_set_fps(p: PAVS_VideoInfo; numerator: Cardinal; denominator: Cardinal);
var x, y, t: Cardinal;
begin
  x := numerator;
  y := denominator;

  while (y > 0) do
  begin
    t := x mod y;
    x := y;
    y := t;
  end;

  p^.fps_numerator := numerator div x;
  p^.fps_denominator := denominator div y;
end;

function avs_is_same_colorspace(x: PAVS_VideoInfo; y: PAVS_VideoInfo): Boolean;
begin
  Result := (x^.pixel_type = y^.pixel_type) or (avs_is_yv12(x) and avs_is_yv12(y));
end;

function avs_get_pitch(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): Integer;
begin
  case plane of
    AVS_PLANAR_U, AVS_PLANAR_V: Result := p^.pitchUV;
  else
    Result := p^.pitch;
  end;
end;

function avs_get_row_size(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): Integer;
begin
  case plane of
    AVS_PLANAR_U, AVS_PLANAR_V:
      if p^.pitchUV <> 0 then
        Result := p^.row_size shr 1
      else
        Result := 0;
    AVS_PLANAR_U_ALIGNED, AVS_PLANAR_V_ALIGNED:
      if p^.pitchUV <> 0 then
      begin
        Result := ((p^.row_size + AVS_FRAME_ALIGN - 1) and (not (AVS_FRAME_ALIGN - 1))) shr 1;

        if Result >= p^.pitchUV then
          Result := p^.row_size shr 1;
      end
      else
        Result := 0;
    AVS_PLANAR_Y_ALIGNED:
      begin
        Result := (p^.row_size + AVS_FRAME_ALIGN - 1) and (not (AVS_FRAME_ALIGN - 1));
        if (Result > p^.pitch) then
          Result := p^.row_size;
      end;
  else
    Result := p^.row_size;
  end;
end;

function avs_get_height(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): Integer;
begin
  case plane of
    AVS_PLANAR_U, AVS_PLANAR_V: Result := p^.height shr 1;
  else
    Result := p^.height;
  end;
end;

function avs_get_read_ptr(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): PByte;
begin
  Result := p^.vfb^.data;

  case plane of
    AVS_PLANAR_U: Inc(Result, p^.offsetU);
    AVS_PLANAR_V: Inc(Result, p^.offsetV);
  else
    Inc(Result, p^.offset);
  end;
end;

function avs_is_writable(p: PAVS_VideoFrame): Boolean;
begin
  Result := (p^.refcount = 1) and (p^.vfb^.refcount = 1);
end;

function avs_get_write_ptr(p: PAVS_VideoFrame; plane: Planes = AVS_NOT_PLANAR): PByte;
begin
  Result := p^.vfb^.data;

  if (plane = AVS_PLANAR_Y) and avs_is_writable(p) then
  begin
    Inc(p^.vfb^.sequence_number);
    Inc(Result, p^.offset);
  end
  else if (plane = AVS_PLANAR_Y) then
    Result := nil
  else
    case (plane) of
      AVS_PLANAR_U: Inc(Result, p^.offsetU);
      AVS_PLANAR_V: Inc(Result, p^.offsetV);
    else
      Inc(Result, p^.offset);
    end;
end;

function avs_defined(v: AVS_Value): Boolean;
begin
  Result := v.vtype <> 'v';
end;

function avs_is_clip(v: AVS_Value): Boolean;
begin
  Result := v.vtype = 'c';
end;

function avs_is_bool(v: AVS_Value): Boolean;
begin
  Result := v.vtype = 'b';
end;

function avs_is_int(v: AVS_Value): Boolean;
begin
  Result := v.vtype = 'i';
end;

function avs_is_float(v: AVS_Value): Boolean;
begin
  Result := (v.vtype = 'i') or (v.vtype = 'f');
end;

function avs_is_string(v: AVS_Value): Boolean;
begin
  Result := v.vtype = 's';
end;

function avs_is_array(v: AVS_Value): Boolean;
begin
  Result := v.vtype = 'a';
end;

function avs_is_error(v: AVS_Value): Boolean;
begin
  Result := v.vtype = 'e';
end;

function avs_as_bool(v: AVS_Value): LongBool;
begin
  Result := v.vboolean;
end;

function avs_as_int(v: AVS_Value): Integer;
begin
  Result := v.vinteger;
end;

function avs_as_string(v: AVS_Value): PChar;
begin
  if avs_is_error(v) or avs_is_string(v) then
    Result := v.vstring
  else
    Result := nil;
end;

function avs_as_float(v: AVS_Value): Single;
begin
  if avs_is_int(v) then
    Result := v.vinteger
  else
    Result := v.vfloating_pt;
end;

function avs_as_error(v: AVS_Value): PChar;
begin
  if avs_is_error(v) then
    Result := v.vstring
  else
    Result := nil;
end;

function avs_as_array(v: AVS_Value): PAVS_Value_array;
begin
  Result := v.varray;
end;

function avs_array_size(v: AVS_Value): Integer;
begin
  if avs_is_array(v) then
    Result := v.array_size
  else
    Result := 1;
end;

function avs_array_elt(v: AVS_Value; index: Integer): AVS_Value;
begin
  if avs_is_array(v) then
    Result := v.varray^[index]
  else
    Result := v;
end;

function avs_new_value_bool(v0: LongBool): AVS_Value;
begin
  Result.vtype := 'b';
  Result.vboolean := v0;
end;

function avs_new_value_int(v0: Integer): AVS_Value;
begin
  Result.vtype := 'i';
  Result.vinteger := v0;
end;

function avs_new_value_string(v0: PChar): AVS_Value;
begin
  Result.vtype := 's';
  Result.vstring := v0;
end;

function avs_new_value_float(v0: Single): AVS_Value;
begin
  Result.vtype := 'f';
  Result.vfloating_pt := v0;
end;

function avs_new_value_error(v0: PChar): AVS_Value;
begin
  Result.vtype := 'e';
  Result.vstring := v0;
end;

function avs_new_value_clip(v0: PAVS_Clip): AVS_Value;
begin
  avs_set_to_clip(Result, v0);
end;

function avs_new_value_array(v0: PAVS_Value_array; size: Integer): AVS_Value;
begin
  Result.vtype := 'a';
  Result.varray := v0;
  Result.array_size := size;
end;

end.

